@@ -37,7 +37,6 @@ import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
-import java.util.concurrent.TimeUnit;
 import java.util.function.BiFunction;
 import java.util.logging.Level;
 import java.util.logging.Logger;
@@ -650,10 +649,15 @@ public class RemoteWebDriver
         Map.of("authenticatorId", authenticator.getId()));
   }
 
+  @Override
+  public boolean isDownloadsEnabled() {
+    return HasDownloads.isDownloadsEnabled(capabilities);
+  }
+
   /**
-   * Retrieves the downloadable files as a map of file names and their corresponding URLs.
+   * Retrieves the names of the downloadable files.
    *
-   * @return A map containing file names as keys and URLs as values.
+   * @return A list containing the names of the downloadable files.
    * @throws WebDriverException if capability to enable downloads is not set
    */
   @Override
@@ -869,6 +873,9 @@ public class RemoteWebDriver
 
     @Override
     public void deleteCookieNamed(String name) {
+      if (name == null || name.isBlank()) {
+        throw new IllegalArgumentException("Cookie name cannot be empty");
+      }
       execute(DriverCommand.DELETE_COOKIE(name));
     }
 
@@ -927,6 +934,9 @@ public class RemoteWebDriver
 
     @Override
     public Cookie getCookieNamed(String name) {
+      if (name == null || name.isBlank()) {
+        throw new IllegalArgumentException("Cookie name cannot be empty");
+      }
       Set<Cookie> allCookies = getCookies();
       for (Cookie cookie : allCookies) {
         if (cookie.getName().equals(name)) {
@@ -949,12 +959,6 @@ public class RemoteWebDriver
 
     protected class RemoteTimeouts implements Timeouts {
 
-      @Deprecated
-      @Override
-      public Timeouts implicitlyWait(long time, TimeUnit unit) {
-        return implicitlyWait(Duration.ofMillis(unit.toMillis(time)));
-      }
-
       @Override
       public Timeouts implicitlyWait(Duration duration) {
         execute(DriverCommand.SET_IMPLICIT_WAIT_TIMEOUT(duration));
@@ -969,18 +973,6 @@ public class RemoteWebDriver
         return Duration.ofMillis(timeout);
       }
 
-      @Deprecated
-      @Override
-      public Timeouts setScriptTimeout(long time, TimeUnit unit) {
-        return setScriptTimeout(Duration.ofMillis(unit.toMillis(time)));
-      }
-
-      @Deprecated
-      @Override
-      public Timeouts setScriptTimeout(Duration duration) {
-        return scriptTimeout(duration);
-      }
-
       @Override
       public Timeouts scriptTimeout(Duration duration) {
         execute(DriverCommand.SET_SCRIPT_TIMEOUT(duration));
@@ -995,12 +987,6 @@ public class RemoteWebDriver
         return Duration.ofMillis(timeout);
       }
 
-      @Deprecated
-      @Override
-      public Timeouts pageLoadTimeout(long time, TimeUnit unit) {
-        return pageLoadTimeout(Duration.ofMillis(unit.toMillis(time)));
-      }
-
       @Override
       public Timeouts pageLoadTimeout(Duration duration) {
         execute(DriverCommand.SET_PAGE_LOAD_TIMEOUT(duration));
@@ -1148,12 +1134,16 @@ public class RemoteWebDriver
         // simulate search by name
         String original = getWindowHandle();
         for (String handle : getWindowHandles()) {
-          switchTo().window(handle);
-          if (windowHandleOrName.equals(executeScript("return window.name"))) {
-            return RemoteWebDriver.this; // found by name
+          try {
+            execute(DriverCommand.SWITCH_TO_WINDOW(handle));
+            if (windowHandleOrName.equals(executeScript("return window.name"))) {
+              return RemoteWebDriver.this; // found by name
+            }
+          } catch (NoSuchWindowException nswe) {
+            // swallow
           }
         }
-        switchTo().window(original);
+        execute(DriverCommand.SWITCH_TO_WINDOW(original));
         throw nsw;
       }
     }
@@ -1243,7 +1233,7 @@ public class RemoteWebDriver
           Stream.concat(
                   credential.toMap().entrySet().stream(),
                   Stream.of(Map.entry("authenticatorId", id)))
-              .collect(Collectors.toUnmodifiableMap((e) -> e.getKey(), (e) -> e.getValue())));
+              .collect(Collectors.toUnmodifiableMap(Map.Entry::getKey, Map.Entry::getValue)));
     }
 
     @Override
